#include "grbc/ninja.h"
#include "grbc/generator.h"
#include "grbc/helpers.h"
#include "grbc/spec.h"
#include "grbc/state.h"

std::string ninja_build_rule_compile_cxx() {
  std::string compiler_path = GState::get().current_platform.cxx_compiler;

  std::string result =
      "## build_rule_compile_cxx ##\n\ncxx_path = " + compiler_path + "\n";
  result += "rule cxx\n";
  result += "  command = $cxx_path -MMD -MT $out -MF $out.d $p_cflags -c $in "
            "-o $out\n";
  result += "  description = Compiling C++ object $in\n";
  result += "  depfile = $out.d\n";
  result += "  deps = gcc\n";
  result += "\n";

  return result;
}

std::string ninja_build_rule_compile_cc() {
  std::string compiler_path = GState::get().current_platform.cc_compiler;

  std::string result =
      "## build_rule_compile_cc ##\n\ncc_path = " + compiler_path + "\n";
  result += "rule cc\n";
  result += "  command = $cc_path -MMD -MT $out -MF $out.d $p_cflags -c $in "
            "-o $out\n";
  result += "  description = Compiling C object $in\n";
  result += "  depfile = $out.d\n";
  result += "  deps = gcc\n";
  result += "\n";

  return result;
}

std::string ninja_build_rule_link_cxx() {
  std::string result = "## build_rule_link_cxx ##\n\n";
  result += "rule link_cxx\n";
  result += "  command = $cxx_path $p_cflags -o $out $in $p_linker_flags\n";
  result += "  description = Linking C++ executables $out\n\n";

  return result;
}

std::string ninja_build_rule_link_cc() {
  std::string result = "## build_rule_link_cc ##\n\n";
  result += "rule link_cc\n";
  result += "  command = $cc_path $p_cflags -o $out $in $p_linker_flags\n";
  result += "  description = Linking C executables $out\n\n";

  return result;
}

std::string
ninja_build_rule_compile_file(const GeneratorCompileCommand &compile_cmd) {
  std::string result = "## Compile: " + compile_cmd.source_file + " ##\n\n";

  std::string compiler_rule = "cxx";

  if (compile_cmd.language_type == LanguageType_C)
    compiler_rule = "cc";

  result += "build $builddir/" + compile_cmd.object_file + ": " +
            compiler_rule + " " + compile_cmd.source_file + "\n";
  result += "   p_cflags = " + compile_cmd.compiler_flags + "\n\n";

  return result;
}

std::string ninja_build_default_variables() {
  std::string result = "## Default variables ##\n\n";
  result += "builddir = " + grbc_get_config().build_dir + "\n";

  result += "\n"; // keep as the last line

  return result;
}

std::string ninja_build_rule_link_exe_target(
    const GeneratorLinkExecutableCommand &link_cmd) {
  std::string result = "## Link: " + link_cmd.output_exe + " ##\n\n";

  std::string file_list;

  for (auto &file : link_cmd.object_files) {
    file_list += "$builddir/" + file + " ";
  }

  file_list.pop_back();

  result += "build " + link_cmd.output_exe + ":  link_cxx " + file_list + "\n";
  result += "  p_linker_flags = " + link_cmd.linker_flags + "\n";
  result += "  p_cflags = \n\n";

  return result;
}

// Ninja generator entry

GeneratorResult ninja_generator() {
  GeneratorResult result{};

  log_msg("running ninja final generation...");

  result.file_name = "build.ninja";

  result.content += "### GRBC BUILT-IN NINJA GENERATOR ###\n\n";
  result.content += ninja_build_default_variables();
  result.content += ninja_build_rule_compile_cc();
  result.content += ninja_build_rule_compile_cxx();
  result.content += ninja_build_rule_link_cc();
  result.content += ninja_build_rule_link_cxx();

  for (auto &target : GState::get().targets) {
    for (auto &compile_cmd : target.compile_commands) {
      result.content += ninja_build_rule_compile_file(compile_cmd);
    }

    for (auto &link_cmd : target.link_executable_commands) {
      result.content += ninja_build_rule_link_exe_target(link_cmd);
    }
  }

  return result;
}